import 'dart:async';
import 'dart:io';

import 'package:ft/adapters/shell.dart';
import 'package:ft/models/platform.dart';
import 'package:ft/models/runtime_exception.dart';
import 'package:path/path.dart' as p;

class Simulator {
  final Platform platform;
  final String os;
  final List<String> permissions;
  final bool headless;
  final String name;
  final String deviceIdentifier;
  late String _id;

  Simulator(
      {required this.platform,
      required this.os,
      required this.deviceIdentifier,
      required this.permissions,
      required this.headless,
      required this.name}) {
    _id = platform.prepareApp(deviceIdentifier, os, permissions, headless, name,
        platform.getBundleName());
  }

  Future<bool> runTests(String fileName, bool record,
      [String outputPath = '/']) async {
    var res = false;
    print('Will record video? $record');
    if (record) {
      var recordingProcess = await Process.start('xcrun', [
        'simctl',
        'io',
        _id,
        'recordVideo',
        p.join(outputPath, '${fileName.split("/").last.split(".").first}.mp4')
      ]);
      print('Executing flutter test');
      try {
        await runAsync('flutter', ['test', '-d', _id, '--pub', fileName]);
        recordingProcess.kill(ProcessSignal.sigint);

        return true;
      } on RuntimeException catch (e) {
        recordingProcess.kill(ProcessSignal.sigint);
        return false;
      }
    }
    print('Running flutter test');
    try {
      await runAsync('flutter', ['test', '-d', _id, '--pub', fileName]);
      return true;
    } on RuntimeException catch (e) {
      return false;
    }
  }

  void cleanUp() {
    return platform.cleanUp(_id);
  }
}
